/************************************************************************************
 * Projet   : DoublesPointeurs
 * Classe   : circ_list.c
 * Auteur   : Ottavio Buonomo
 * Desc.    : Liste générique simplement chainée
 * Version  : 1.0
 * Date     : 12.08.2019
 ***********************************************************************************/

#include <stdlib.h>
#include "circ_list.h"

/// Création de la liste
/// \return nouvelle liste
Element* list_create() {
    Element *element = malloc(sizeof(Element));
    element->content = NULL;
    element->next = NULL;
    return element;
}

/// Vérifie si la liste est vide
/// \param head - entrée de la liste
/// \return
bool list_empty(Element* head) {
    return head->next == NULL;
}

/// Longueur de la liste
/// \param head - entrée de la liste
/// \return longueure totale de la liste
unsigned int list_count(Element* head) {
    int cpt = 0;
    while (head->next) {
        cpt++;
        head = head->next;
    }
    return cpt;
}

/// Déplace un élément de la liste
/// \param head - entrée de la liste
/// \return
Element* list_move(Element* head) {
    Element *element;
    element = head->next;
    return element;
}

/// Inser un élément après un autre
/// \param head - élément principal
/// \param data - élément à ajouter
/// \return nouvelle liste
Element* list_insert_after(Element* head, void* data) {
    Element *element = malloc(sizeof(Element));
    element->content = data;
    element->next = head->next;
    head->next = element;
    return element;
}

/// Inser un élément avant un autre
/// \param head - élément principal
/// \param data - élément à ajouter
/// \return nouvelle liste
Element* list_insert_before(Element* head, void* data) {
    Element *element = malloc(sizeof(Element));
    element->next = head;
    element->content = data;
    return element;
}

/// Comparaison de deux éléments
/// \param this - élément 1
/// \param other - élément 2
/// \return 1 ou 0
int compare(void* this, void* other) {
    return this == other;
}

/// Recherche dans la liste
/// \param head -  entrée de la liste
/// \param data - objet à chercher
/// \param compare - valeur de comparaison
/// \return élément recherché
Element* list_search(Element* head, void* data, int (*compare)(void*, void*)) {
    while (head->next) {
        if (compare(head->content, data)) {
            return head;
        }else{
            head = head->next;
        }
    }
    return NULL;
}
/// Supprime un élément de la liste
/// \param head - entrée de la liste
/// \param data - élément qu'on veut supprimer
/// \param compare - valeur de compariason
/// \return élément supprimé
Element* list_remove(Element* head, void* data, int (*compare)(void*, void*)) {
    int cpt = 0;
    while (head->next) {
        if (cpt == 0) {
            if (compare(head->content, data)) {
                return head;
            }
            cpt++;
        } else {
            if (compare(head->next->content, data)) {
                void* tmp = head->next;
                head->next = head->next->next;
                return tmp;
            }
        }
        head = head->next;
    }
    return NULL;
}

/// Libère une donnée
/// \param data - donnée à libèrer
void data_free(void* data) {
    free(data);
}

/// Libère la liste
/// \param head - entrée de la liste
/// \param data_free - fonction de libération
void list_free(Element* head, void (*data_free)(void*)) {
    while (head->next) {
        head = head->next->next;
        data_free(head->next);
    }
    data_free(head);
}