/************************************************************************************
 * Projet   : DoublesPointeurs
 * Classe   : martix.c
 * Auteur   : Ottavio Buonomo
 * Desc.    : Représentation de matrice plus quelques opérations
 * Version  : 1.0
 * Date     : 12.08.2019
 ***********************************************************************************/

#include <stdlib.h>
#include "matrix.h"

/// Création d'une nouvelle matrice
/// \param row - nb de lignes
/// \param col - nb de colonnes
/// \return nouvelle matrice
m_double* matrix_alloc(unsigned int row, unsigned int col) {
    m_double *matrix = malloc(sizeof(m_double));
    matrix->row = row;
    matrix->col = col;
    double **line = malloc(sizeof(double*) * row);
    matrix->content = line;
    for (int i = 0; i < row; ++i) {
        double *column = malloc(sizeof(double) * col);
        matrix->content[i] = column;
    }
    return matrix;
}

/// Transpose une matrice
/// \param m - matrice à transposer
/// \return matrice m transposée
m_double* matrix_transpose(m_double* m) {
    m_double *transpose = matrix_alloc(m->col, m->row);
    for (int i = 0; i < transpose->row; ++i) {
        for (int j = 0; j < transpose->col; ++j) {
            transpose->content[i][j] = m->content[j][i];
        }
    }
    return transpose;
}

/// Addition des deux matrices en paramètes
/// \param m1 - matrice 1
/// \param m2 - matrice 2
/// \return addition ou NULL si l'addition est impossible
m_double* matrix_add(m_double* m1, m_double* m2) {
    if ((m1->row == m2->row) && (m1->col == m2->col)) {
        m_double *add = matrix_alloc(m1->row, m2->col);
        for (int i = 0; i < m1->row; ++i) {
            for (int j = 0; j < m1->col; ++j) {
                add->content[i][j] = m1->content[i][j] + m2->content[i][j];
            }
        }
        return add;
    } else {

        return NULL;
    }
}

/// Multiplication des deux matrices en paramètres
/// \param m1 - matrice 1
/// \param m2 - matrice 2
/// \return multiplication ou NULL si la multiplication est impossible
m_double* matrix_mult(m_double* m1, m_double* m2) {
    if (m1->row == m2->col) {
        m_double *mult = matrix_alloc(m1->row, m2->col);
        double sum = 0.0;
        for (int i = 0; i < m1->row; ++i) {
            for (int j = 0; j < m2->col; ++j) {
                for (int k = 0; k < m2->row; ++k) {
                    sum = sum + m1->content[i][k] * m2->content[k][j];
                }
                mult->content[i][j] = sum;
                sum = 0.0;
            }
        }
        return mult;
    } else {
        return NULL;
    }
}