/************************************************************************************
 * Projet   : DoublesPointeurs
 * Classe   : martix_compute.c
 * Auteur   : Ottavio Buonomo
 * Desc.    : Opérations sur des matrices
 * Version  : 1.0
 * Date     : 12.08.2019
 ***********************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "matrix.h"

#define LENGTH 256

/// Affiche une matrice
/// \param m - matrice
void printMatrix(m_double *m) {
    printf("%d %d\n", m->row, m->col);
    for (int i = 0; i < m->row; ++i) {
        for (int j = 0; j < m->col; ++j) {
            printf("%g ", m->content[i][j]);
        }
        printf("\n");
    }
}

/// Récupération d'une matrice à partir d'un fichier
/// \param name - nom du fichier
/// \return matrice provenant du fichier
m_double* readFile(char name[80]) {
    m_double *matrix;

    char line[LENGTH];
    FILE *fp;

    // Open file for reading
    fp = fopen(name,"r");
    if (!fp) {
        fprintf(stderr,"file not found: %s\n", name);
        return 0;
    }

    // Read and print each line in the file
    char sep[] = " ";
    int nb_fields = 0;
    int line_cnt = 0;

    while ((fgets(line, LENGTH, fp)) != NULL ) {

        if (line_cnt == 0) {
            nb_fields = 2;
        } else {
            nb_fields = matrix->col;
        }

        char *token;
        // Point to first token
        token = strtok(line, sep);
        // Iterate through other tokens and store them in fields
        int cnt = 0;
        char *fields[nb_fields];

        while (token) {
            fields[cnt] = token;
            token = strtok(NULL, sep);
            cnt++;
        }

        if (line_cnt == 0) {
            matrix = matrix_alloc(atoi(fields[0]), atoi(fields[1]));
        } else {
            for (int i = 0; i < nb_fields; ++i) {
                matrix->content[line_cnt - 1][i] = atoi(fields[i]);
            }
        }

        line_cnt++;
    }

    // Close the file
    fclose(fp);

    return matrix;
}

int main(int argc, char *argv[]) {
    char *operation = argv[1];
    char *fileName1 = argv[2];
    char *fileName2 = argv[3];

    if (!strcmp("add", operation)) {
        m_double *matA = readFile(fileName1);
        m_double *matB = readFile(fileName2);
        if (!matrix_add(matA, matB)) {
            exit(1);
        } else {
            printMatrix(matrix_add(matA, matB));
        }
    } else if (!strcmp("mult", operation)) {
        m_double *matA = readFile(fileName1);
        m_double *matB = readFile(fileName2);
        if (!matrix_mult(matA, matB)) {
            exit(1);
        } else {
            printMatrix(matrix_mult(matA, matB));
        }
    } else if (!strcmp("transpose", operation)) {
        m_double *matA = readFile(fileName1);
        printMatrix(matrix_transpose(matA));
    } else {
        printf("%s is not an operation !", operation);
        exit(1);
    }

    return 0;
}
